@@ -13,6 +13,7 @@ const webex_buttons = {check_hidden:          "<b>&check;</b>",
                        question_previous:     "",
                        question_previous_alt: ""}
 
+
 /* update total correct if #webex-total_correct exists */
 update_total_correct = function() {
   console.log("webex: update total_correct");
@@ -35,7 +36,7 @@ update_total_correct = function() {
 check_func = function() {
   console.log("webex: check answer");
 
-  //var cl = this.parentElement.classList;
+  //var cl = elem.parentElement.classList;
   var cl = this.closest(".webex-box").classList;
   if (cl.contains("unchecked")) {
     cl.remove("unchecked");
@@ -48,6 +49,12 @@ check_func = function() {
   }
 }
 
+/* helper functions for checking the webex ID */
+const id_checker = {"fmt": new RegExp("[g-zG-Z]"),
+                    "load": function(e) { return e.getAttribute("id").match("(?<=(-)).*$")[0]; },
+                    "eval": new TextEncoder(),
+                    "dval": new TextDecoder()}
+
 /* Show/hide correct solution */
 solution_func = function() {
   console.log("webex: show/hide solution");
@@ -64,38 +71,60 @@ solution_func = function() {
 
 /* function to check if the real answer is numeric */
 convert_to_numeric = function(x) {
-    if (typeof x == "string") {
-        /* do nothing */
-    } else if (x.length == 1) {
-        /* take first element */
-        x = x[0]
-    } else {
-        return NaN;
-    }
-
-    /* remove spaces for easier parsing */
-    x = x.replace(/\s/g, "");
-
-    /* Define patterns for different formats, note that spaces have been removed above */
-    const patterns = [
-        {regex: /^[+-]?\d+(\.\d{3})*,\d+$/, decimal: ",", thousand: "." },  // Format: "1.100.100.100,3"
-        {regex: /^[+-]?\d+(,\d{3})*\.\d+$/, decimal: ".", thousand: "," },  // Format: "1,100,100,100.3"
-        {regex: /^[+-]?\d+(\.\d+)?$/, decimal: "." },                       // Format: "1100100100.5"
-        {regex: /^[+-]?\d+,\d+$/, decimal: "," }                            // Format: "1100100100,5"
-    ];
-
-    /* testing all regular expressions, convert to float if possible */
-    for (const { regex, decimal, thousand } of patterns) {
-        if (regex.test(x)) {
-            let numeric = x;
-            if (thousand) numeric = numeric.replace(new RegExp(`\\${thousand}`, "g"), "");
-            numeric = numeric.replace(decimal, ".");
-            return parseFloat(numeric);
-        }
-    }
-    
-    /* input of length 1 but none of the known formats, return NaN */
+  if (typeof x == "string") {
+    /* do nothing */
+  } else if (x.length == 1) {
+    /* take first element */
+    x = x[0]
+  } else {
     return NaN;
+  }
+
+  /* remove spaces for easier parsing */
+  x = x.replace(/\s/g, "");
+
+  /* Define patterns for different formats, note that spaces have been removed above */
+  const patterns = [
+    {regex: /^[+-]?\d+(\.\d{3})*,\d+$/, decimal: ",", thousand: "." },  // Format: "1.100.100.100,3"
+    {regex: /^[+-]?\d+(,\d{3})*\.\d+$/, decimal: ".", thousand: "," },  // Format: "1,100,100,100.3"
+    {regex: /^[+-]?\d+(\.|\.\d+)?$/, decimal: "." },                    // Format: "1100100100.5" or "1100100100."
+    {regex: /^[+-]?\d+(,|,\d+)$/, decimal: "," }                        // Format: "1100100100,5" or "1100100100,"
+  ];
+
+
+  /* testing all regular expressions, convert to float if possible */
+  for (const { regex, decimal, thousand } of patterns) {
+    if (regex.test(x)) {
+      let numeric = x;
+      if (thousand) numeric = numeric.replace(new RegExp(`\\${thousand}`, "g"), "");
+      numeric = numeric.replace(decimal, ".");
+      return parseFloat(numeric);
+    }
+  }
+
+  /* input of length 1 but none of the known formats, return NaN */
+  return NaN;
+}
+
+/* Checking webex ID, returns formalsol */
+check_id = function(e) {
+  /* loading webex ID */
+  var id = id_checker["load"](e);
+
+  /* extracting answer */
+  let answer = e.dataset.answer;
+
+  /* checking unique id, prepare and return array */
+  if (!id_checker["fmt"].test(id)) {
+    const a = id_checker["eval"].encode(atob(answer));
+    const b = id_checker["eval"].encode(id);
+    const res = a.map((byte, index) => byte^b[index % b.length]);
+    answer = id_checker["dval"].decode(res);
+    /* Replacing &amp; with ' before parsing */
+    answer = answer.replaceAll("&apos;", "'");
+  }
+
+  return JSON.parse(answer);
 }
 
 /* function for checking solveme answers */
@@ -114,6 +143,9 @@ solveme_func = function(e) {
   var cl = this.classList;
   var my_answer = this.value;
 
+  /* extracting classes */
+  var cl = this.classList;
+
   /* empty answer? Job done */
   if (my_answer == "") {
     cl.remove("webex-correct");
@@ -121,50 +153,49 @@ solveme_func = function(e) {
     return false;
   }
 
-  /* "Else" we continue evaluating the answer */
-  var real_answers = JSON.parse(this.dataset.answer);
+  /* Find closest (parent) webex-question */
+  let formalsols = check_id(this)
 
   /* by default we assume the users' answer is incorrect */
   var user_answer_correct = false;
 
   /* check if the correct answer is numeric, i.e. if 
-   * real_answers is of length 1 containing one single numeric
+   * formalsols is of length 1 containing one single numeric
    * value in a known format, else, NaN is returned */
-  const num_real_answer = convert_to_numeric(real_answers);
-  const num_my_answer   = convert_to_numeric(my_answer);
+  const num_formalsol = convert_to_numeric(formalsols);
+  const num_my_answer = convert_to_numeric(my_answer);
 
   /* if the correct answer is numeric (float), the user's answer
    * must also be numeric. If not, it is wrong. Else we can
    * compare floating point numbers */
-  if (!isNaN(num_real_answer) && !isNaN(num_my_answer)) {
-    //DEV// console.log("webex: evaluating numeric answer")
+  if (!isNaN(num_formalsol) && !isNaN(num_my_answer)) {
     /* check if the real answer and the user input are numerically the same;
      * adding 'delta' to avoid precision issues */
-    var diff = Math.abs(num_real_answer - num_my_answer);
+    var diff = Math.abs(num_formalsol - num_my_answer);
     if (diff < parseFloat(this.dataset.tol) + eps) { user_answer_correct = true; }
 
   /* if the question contains regex, a regular expression is used
-   * to evaluate the users answer (only possible if length of answers is 1) */
-  } else if (cl.contains("regex") && real_answers.length == 1) {
-    //console.log("webex: evaluating answer using regular expression")
-    let regex = new RegExp(real_answers[0], cl.contains("ignorecase") ? "i" : "");
-    if (regex.test(my_answer)) { user_answer_correct = true; }
+   * to evaluate the users answer. Allows for multiple regular expressions */
+  } else if (cl.contains("regex")) {
+    for (let i = 0; i < formalsols.length; i++) {
+        let regex = new RegExp(formalsols[i], cl.contains("ignorecase") ? "i" : "");
+        if (regex.test(my_answer)) { user_answer_correct = true; break; }
+    }
 
   /* else we evaluate on 'string level', considering the creators preferences
    * regarding set options */
   } else {
-    //console.log("webex: evaluating string answer")
     /* modify/prepare answer */
     if (cl.contains("ignorecase")) { my_answer = my_answer.toLowerCase(); }
     if (cl.contains("nospaces"))   { my_answer = my_answer.replace(/ /g, ""); }
 
     /* if the real answer includes user input - correct */
-    if (real_answers.includes(my_answer)) {
+    if (formalsols.includes(my_answer)) {
       user_answer_correct = true;
   
       // added regex bit
       if (cl.contains("regex")) {
-        answer_regex = RegExp(real_answers.join("|"))
+        answer_regex = RegExp(formalsols.join("|"))
         if (answer_regex.test(my_answer)) {
           cl.add("webex-correct");
         }
@@ -188,15 +219,25 @@ solveme_func = function(e) {
 select_func = function(e) {
   console.log("webex: check select");
 
-  var cl = this.classList
+  var options   = [];
+  Array.from(this.querySelectorAll("option")).forEach((option, index) => {
+    if (!option.hasAttribute("value") || !option.getAttribute("value") === "blank") {
+      options.push(option);
+    }
+  });
 
-  /* add style */
-  cl.remove("webex-incorrect");
-  cl.remove("webex-correct");
-  if (this.value == "answer") {
-    cl.add("webex-correct");
-  } else if (this.value != "blank") {
-    cl.add("webex-incorrect");
+  /* get selected option */
+  function get_selected(options) { return options.findIndex(option => option.selected); }
+  var option_index = get_selected(options)
+
+  /* loading webex ID */
+  var formalsol = check_id(this)
+
+  /* add class for current selection */
+  this.classList.remove("webex-incorrect");
+  this.classList.remove("webex-correct");
+  if (option_index >= 0) {
+    this.classList.add(formalsol[option_index] === 1 ? "webex-correct" : "webex-incorrect");
   }
 
   update_total_correct();
@@ -206,21 +247,31 @@ select_func = function(e) {
 radiogroups_func = function(e) {
   console.log("webex: check radiogroups");
 
-  var checked_button = document.querySelector("input[name=" + this.id + "]:checked");
-  var cl = checked_button.parentElement.classList;
-  var labels = checked_button.parentElement.parentElement.children;
+  /* get real answer (binary integer array) */
+  var id = e.target.getAttribute("name");
+  var group = document.querySelector(".webex-radiogroup[id='webex-" + id + "']");
+  var formalsol = check_id(group)
 
-  /* get rid of styles */
-  for (i = 0; i < labels.length; i++) {
-    labels[i].classList.remove("webex-incorrect");
-    labels[i].classList.remove("webex-correct");
+  /* check which radiobuttion is selected (user answer) */
+  function get_checked(radios) {
+    return radios.findIndex(radios => radios.checked);
   }
+  var radios = Array.from(group.querySelectorAll("input[type='radio']"))
+  var radio_index = get_checked(radios);
 
-  /* add style */
-  if (checked_button.value == "answer") {
-    cl.add("webex-correct");
+  /* remove existing classes */
+  radios.forEach(radio => {
+    let cl = radio.closest("label").classList
+    cl.remove("webex-incorrect");
+    cl.remove("webex-correct");
+  });
+
+  /* add class for current selection */
+  let cl = radios[radio_index].closest("label").classList
+  if (formalsol[radio_index] === 1) {
+      cl.add("webex-correct");
   } else {
-    cl.add("webex-incorrect");
+      cl.add("webex-incorrect");
   }
 
   update_total_correct();
@@ -231,18 +282,29 @@ radiogroups_func = function(e) {
 checkboxgroups_func = function(e) {
   console.log("webex: check checkboxgroups");
 
-  /* list of all answer elements (correct and incorrect) */
-  var inputs = document.querySelectorAll("div[id='" + this.id + "'] input")
+  /* get real answer (binary integer array) */
+  var group = document.querySelector(".webex-checkboxgroup[id='" + this.id + "']");
+  var formalsol = check_id(group);
 
-  /* setting class for correct/incorrect answers */
-  inputs.forEach(function(input) {
-      var label = input.parentNode
-      if ((input.checked && input.value == "answer") || (!input.checked && input.value == "")) {
-          //input.setAttribute("class", "webex-correct")
-          label.setAttribute("class", "webex-correct")
-      } else {
-          label.setAttribute("class", "webex-incorrect")
-      }
+  /* check which checkbox is checked (user answer) */
+  function get_checked(group) {
+      const checkboxes = Array.from(group.querySelectorAll("input[type='checkbox']"));
+      return checkboxes.reduce((indices, checkbox, index) => {
+          if (checkbox.checked) indices.push(index);
+          return indices;
+      }, []);
+  }
+  var checks = Array.from(group.querySelectorAll("input[type='checkbox']"));
+  var check_index = get_checked(group);
+
+  checks.forEach((e, index) => {
+    var label = e.parentNode;
+    var ans = formalsol[index];
+    if ((e.checked && ans === 1) || (!e.checked && ans === 0)) {
+        label.setAttribute("class", "webex-correct")
+    } else {
+        label.setAttribute("class", "webex-incorrect")
+    }
   });
 
   update_total_correct();
@@ -265,7 +327,7 @@ shuffle_array = function(x) {
  * ---------------------------------------------------------
  * --------------------------------------------------------- */
 window.onload = function() {
-  console.log("webex onload");
+  //console.log("webex: onload");
 
   /* setting up buttons and actions to show/hide answers */
   document.querySelectorAll(".webex-check").forEach(section => {
@@ -314,10 +376,6 @@ window.onload = function() {
     solveme.setAttribute("spellcheck", "false");
     solveme.value = "";
 
-    /* adjust answer for ignorecase or nospaces */
-    if (solveme.classList.contains("ignorecase")) {
-      solveme.dataset.answer = solveme.dataset.answer.toLowerCase();
-    }
     /* adjust answer for 'no spaces' (ignore spaces) */
     if (solveme.classList.contains("nospaces")) {
       solveme.dataset.answer = solveme.dataset.answer.replace(/ /g, "");
@@ -361,7 +419,7 @@ window.onload = function() {
 
       /* current question/position */
       let currentPosition = parseInt(group.dataset.currentPosition);
-  
+
       /* Hide the current question */
       questions.forEach(question => { question.classList.remove("active"); });
 
@@ -370,28 +428,29 @@ window.onload = function() {
       if (currentPosition < 0) currentPosition = currentPosition + questionOrder.length
 
       /* Display the new question */
-      // devel // console.log("set question " + questionOrder[currentPosition] +
-      // devel //             " (" + currentPosition + ") as active");
       questions[questionOrder[currentPosition]].classList.add("active");
-  
+
       // Update the currentPosition data attribute on the group div
       group.dataset.currentPosition = currentPosition;
     };
   }
 
-  
   document.querySelectorAll(".webex-group").forEach(group => {
     const questions = Array.from(group.querySelectorAll(".webex-question"));
-    const questionOrder = shuffle_array(questions.length);
+
+    /* Create vector with question order. If the webex-group does
+     * not have class '.noshuffle' we will randomly shuffle the order */
+    let questionOrder = Array.from({length: questions.length}, (v, i) => i);
+    if (!group.classList.contains("noshuffle")) {
+      questionOrder = shuffle_array(questionOrder);
+    }
 
     /* take start position (if set) or start at 0 */
     const currentPosition = parseInt(group.getAttribute("data-start-position")) || 0;
 
     /* show the default question for each group */
     questions[questionOrder[currentPosition]].classList.add("active");
-    // devel // console.log("set question " + questionOrder[currentPosition] +
-    // devel //             " (" + currentPosition + ") as active; " + questionOrder);
-  
+
     /* store random order of questions as well as current position */
     group.dataset.questionOrder   = questionOrder;
     group.dataset.currentPosition = currentPosition;
@@ -417,12 +476,122 @@ window.onload = function() {
         previousButton.addEventListener("click", handleQuestionClick(group, questions, -1));
         li_previous.appendChild(previousButton);
 
-        console.log(button_ul);
         if (webex_buttons.question_previous.length > 0) button_ul.appendChild(li_previous);
         if (webex_buttons.question_next.length > 0) button_ul.appendChild(li_next);
     });
   });
 
+  /* Development options */
+  document.querySelectorAll(".webex-question.check").forEach(question => {
+      question.querySelector(".webex-button-check").click()
+  });
+  document.querySelectorAll(".webex-question.solution").forEach(question => {
+      let btn = question.querySelector(".webex-button-solution");
+      if (btn !== null) { btn.click(); }
+  });
+
+  /* Pre-filling <input> fields */
+  document.querySelectorAll(".webex-question.prefill").forEach(question => {
+      question.querySelectorAll("input.webex-solveme").forEach(solveme => {
+          let x = JSON.parse(solveme.dataset.answer);
+          solveme.value = x[0];
+          /* Trigger the on change event */
+          solveme.dispatchEvent(new Event("change"));
+      });
+
+      /* Pre-filling radiogroups, that is for schoice questions */
+      question.querySelectorAll(".webex-radiogroup").forEach(radiogroup => {
+          let x = JSON.parse(radiogroup.dataset.answer);
+          let idx = x.indexOf(1);
+          let radios = radiogroup.querySelectorAll('input[type="radio"]');
+          radios[idx].checked = true;
+          /* Trigger the on change event */
+          radios[idx].dispatchEvent(new Event("change", { bubbles: true }));
+      });
+
+      /* Pre-filling checkboxgroups, that is for schoice questions w/ dropdown menus */
+      question.querySelectorAll(".webex-select").forEach(dropdown => {
+          let x = JSON.parse(dropdown.dataset.answer);
+          let idx = x.indexOf(1) + 1; /* +1 to skip the first blank option */
+          let options = dropdown.querySelectorAll("option");
+          options[idx].selected = true;
+          /* Trigger the on change event */
+          options[idx].dispatchEvent(new Event("change", { bubbles: true }));
+      });
+
+      /* Pre-filling checkboxgroups, that is for mchoice questions */
+      question.querySelectorAll(".webex-checkboxgroup").forEach(radiogroup => {
+          let x = JSON.parse(radiogroup.dataset.answer);
+          let checkboxes = radiogroup.querySelectorAll('input[type="checkbox"]');
+          for (let i = 0; i < checkboxes.length; i++) {
+              checkboxes[i].checked = x[i];
+          }
+          /* Trigger the on change event */
+          checkboxes[0].dispatchEvent(new Event("change", { bubbles: true }));
+      });
+  });
+
+  /* Set input width when show_tolerance is used */
+  function calc_width(x, txt, offset = 20) {
+      /* We do the following:
+       * Create a new span, insert the value we need in the .tolerance
+       * input node, and calculate its offsetWidth. Then remove it
+       * again; this is just used to calculate the required width. */
+      var cwtmp = document.createElement("span");
+      x.appendChild(cwtmp);
+      cwtmp.innerHTML = txt || "";
+      var w = parseInt(cwtmp.getBoundingClientRect().width) + offset;
+      console.log("width: " + w)
+      cwtmp.remove();
+      return w;
+  }
+
+  /* If tolerance should be shown (devel option) we calculate:
+   * - the width required to properly display the solution
+   * - the width required to properly display the tolerance
+   * - add a new element to display the tolerance
+   * - reduce the width of the solution input box. */
+  document.querySelectorAll(".webex-question.tolerance").forEach(question => {
+      question.querySelectorAll("input[data-tol]").forEach(elem => {
+          var w = null;
+
+          /* Add new element showing the tolerance */
+          let tol = document.createElement("input");
+          tol.type = "text"; // <inpyt type="text">
+          tol.disabled = true; // Disable input element
+          tol.classList.add("tolerance"); // Appending class
+          tol.value = "± " + elem.getAttribute("data-tol");
+
+          /* Add new node */
+          elem.parentNode.insertBefore(tol, elem)
+
+          var body  = document.querySelector("body");
+
+          /* If n > 1 we have invisible input nodes for which we can't
+           * calculate the client's required bounding box (i.e., display
+           * size). Thus, we clone the element, copy it into the body
+           * with an of-view position, calclulate it's width, and remote
+           * it immediately. */
+          let clone = elem.cloneNode(true);
+          clone.style.position = "absolute";
+          clone.style.left = "-1000px";
+          clone.style.top = "-1000px";
+          clone.style.display = "visible";
+          body.appendChild(clone);
+          let clone_w = clone.getBoundingClientRect().width;
+          body.removeChild(clone);
+
+          /* Calculate required width of the 'tolerance' node, and the
+           * reduction of the original 'input' node */
+          w    = calc_width(body, tol.value);
+          wmin = calc_width(body, elem.value);
+          let wnew = Math.max(wmin, clone_w - w);
+
+          /* Set width */
+          tol.style.width  = w + "px";
+          elem.style.width = wnew + "px";
+      });
+  });
 
   update_total_correct();
 }
